@@ -202,12 +202,15 @@ class TransactionReceipt:
 
     @trace_property
     def events(self) -> Optional[EventDict]:
-        if not self.status and self._events is None:
-            self._get_trace()
-            # get events from the trace - handled lazily so that other
-            # trace operations are not blocked in case of a decoding error
-            initial_address = str(self.receiver or self.contract_address)
-            self._events = _decode_trace(self._raw_trace, initial_address)  # type: ignore
+        if self._events is None:
+            if self.status:
+                self._events = _decode_logs(self.logs)  # type: ignore
+            else:
+                self._get_trace()
+                # get events from the trace - handled lazily so that other
+                # trace operations are not blocked in case of a decoding error
+                initial_address = str(self.receiver or self.contract_address)
+                self._events = _decode_trace(self._raw_trace, initial_address)  # type: ignore
         return self._events
 
     @trace_property
@@ -324,7 +327,18 @@ class TransactionReceipt:
         if silent is None:
             silent = self._silent
 
-        return self.sender.transfer(  # type: ignore
+        sender = self.sender
+        if isinstance(sender, EthAddress):
+            # if the transaction wasn't broadcast during this brownie session,
+            # check if the sender is unlocked - we might be able to replace anyway
+            from brownie import accounts
+
+            if sender in accounts:
+                sender = accounts.at(sender)
+            else:
+                raise ValueError("Sender address not in `accounts`")
+
+        return sender.transfer(  # type: ignore
             self.receiver,
             self.value,
             gas_limit=self.gas_limit,
